#!/bin/bash

# ------------------------------------------------------------------------
# TODO
# 
# Redirect all local mail to an external address
# If IPv6 only install IPv4 tunnel
# Create a new bash script which checks logs etc via cron
#
# ------------------------------------------------------------------------

# test is we are running on Ubuntu
if [ $(uname -v | cut -d '-' -f 2 | awk '{print $1}') != "Ubuntu" ];
then
  echo
  echo This script currently only works on Ubuntu
  echo
  exit
fi

if [ $# -eq 0 ]
  then
    echo
    echo $(tput setaf 1)
    echo "ServerSetup will set up a completely fresh Ubuntu VPS from scratch"
    echo "You will be prompted for certain things throughout"
    echo
    echo "Usage: ./UbuntuSetup.sh hostname username (must be run as root)"
    echo "hostname is the desired FQDN of your system and cannot contain spaces"
    echo "username is the username of the new system user that will be set up and added to the root group"
    echo
    echo "This script should be run on a bare Ubuntu system"
    echo $(tput setaf 7)
    exit 1
fi

if [ -z "$1" ]
  then
    echo "hostname can not be an empty string"
    exit 1
fi

if [ -z "$2" ]
  then
    echo "username can not be an empty string"
    exit 1
fi

host=$1
shorthost=${host%%.*}
hostdomain=$(echo $host | cut -d '.' -f 2-)
user=$2

# find ipv4 and ipv6 addresses
myv4=$(ip -4 addr | grep global | awk '{print $2}' | cut -d '/' -f 1)
myv6=$(ip -6 addr | grep global | awk '{print $2}' | cut -d '/' -f 1)

# apply any updates
echo
echo "$(tput setaf 1)Updating system. Please wait..."$(tput setaf 7)
apt-get update
apt-get -option=Dpkg::options::=--force-confmiss --option=Dpkg::options::=--force-confold --assume-yes upgrade

# Install some useful packages
echo
echo "$(tput setaf 1)Installing some needed utilities"$(tput setaf 7)
apt-get -option=Dpkg::options::=--force-confmiss --option=Dpkg::options::=--force-confold --assume-yes install nano ca-certificates apt-transport-https software-properties-common lsb-core cron rsyslog man curl wget pwgen unzip zip dnsutils iputils-ping net-tools nmap
systemctl enable cron

# Harden sysctl

echo "# Protection against IP spoofing" >> /etc/sysctl.conf
echo "net.ipv4.conf.all.rp_filter = 1" >> /etc/sysctl.conf
echo "net.ipv4.conf.default.rp_filter = 1" >> /etc/sysctl.conf
echo "# Ignoring ICMP broadcast requests >>" /etc/sysctl.conf
echo "net.ipv4.icmp_echo_ignore_broadcasts = 1" >> /etc/sysctl.conf
echo "# Disabling source packet routing" >> /etc/sysctl.conf
echo "net.ipv4.conf.all.accept_source_route = 0" >> /etc/sysctl.conf
echo "net.ipv6.conf.all.accept_source_route = 0" >> /etc/sysctl.conf
echo "net.ipv4.conf.default.accept_source_route = 0" >> /etc/sysctl.conf
echo "net.ipv6.conf.default.accept_source_route = 0" >> /etc/sysctl.conf
echo "# Ignoring send redirects" >> /etc/sysctl.conf
echo "net.ipv4.conf.all.send_redirects = 0" >> /etc/sysctl.conf
echo "net.ipv4.conf.default.send_redirects = 0" >> /etc/sysctl.conf
echo "# Ignoring ICMP redirects" >> /etc/sysctl.conf
echo "net.ipv4.conf.all.accept_redirects = 0" >> /etc/sysctl.conf
echo "net.ipv6.conf.all.accept_redirects = 0" >> /etc/sysctl.conf
echo "net.ipv4.conf.default.accept_redirects = 0" >> /etc/sysctl.conf
echo "net.ipv6.conf.default.accept_redirects = 0" >> /etc/sysctl.conf
echo "# Blocking SYN attacks" >> /etc/sysctl.conf
echo "net.ipv4.tcp_syncookies = 1" >> /etc/sysctl.conf
echo "net.ipv4.tcp_max_syn_backlog = 2048" >> /etc/sysctl.conf
echo "net.ipv4.tcp_synack_retries = 2" >> /etc/sysctl.conf
echo "net.ipv4.tcp_syn_retries = 5" >> /etc/sysctl.conf
echo "# Logging Martians" >> /etc/sysctl.conf
echo "net.ipv4.conf.all.log_martians = 1" >> /etc/sysctl.conf
echo "net.ipv4.icmp_ignore_bogus_error_responses = 1" >> /etc/sysctl.conf
echo "# Allow pings" >> /etc/sysctl.conf
echo "net.ipv4.icmp_echo_ignore_all = 0" >> /etc/sysctl.conf
echo "# Congestion: bbr" >> /etc/sysctl.conf
echo "net.core.default_qdisc = fq" >> /etc/sysctl.conf
echo "net.ipv4.tcp_congestion_control = bbr" >> /etc/sysctl.conf
echo "net.ipv4.tcp_notsent_lowat = 16384" >> /etc/sysctl.conf
sysctl -p

# find version of Ubuntu we are using
uver=$(lsb_release -sr)

echo
echo "$(tput setaf 1)Setting hostname"$(tput setaf 7)
hostnamectl set-hostname $host

# Change timezone to UK
echo
echo "$(tput setaf 1)Setting up timezone and locale"$(tput setaf 7)
timedatectl set-timezone Europe/London
apt-get -option=Dpkg::options::=--force-confmiss --option=Dpkg::options::=--force-confold --assume-yes install language-pack-en
update-locale LANG=en_GB.UTF-8 LC_MESSAGES=en_GB.UTF-8

# Tune VPS for network latency
echo
echo "$(tput setaf 1)Setting up tuned utility"$(tput setaf 7)
apt-get -option=Dpkg::options::=--force-confmiss --option=Dpkg::options::=--force-confold --assume-yes install tuned tuned-utils tuned-utils-systemtap
systemctl enable tuned 
systemctl start tuned 
tuned-adm profile network-latency
tuned-adm active

# Add new user and add them to the sudo group
echo
echo "$(tput setaf 1)Setting up new user"$(tput setaf 7)
adduser $user
usermod -aG sudo $user

# Configure and secure SSH (including adding my public key)
echo
echo "$(tput setaf 1)Setting up SSH"$(tput setaf 7)
port=22
port=`shuf -i 49152-65535 -n1`
sed -i -e '/^\(#\|\)Port 22/s/^.*$/Port '"${port}"'/' /etc/ssh/sshd_config
sed -i -e '/^\(#\|\)LogLevel/s/^.*$/LogLevel VERBOSE/' /etc/ssh/sshd_config
sed -i -e '/^\(#\|\)UseDNS/s/^.*$/UseDNS no/' /etc/ssh/sshd_config
sed -i -e '/^\(#\|\)LoginGraceTime/s/^.*$/LoginGraceTime 1m/' /etc/ssh/sshd_config
sed -i -e '/^\(#\|\)PermitRootLogin/s/^.*$/PermitRootLogin no/' /etc/ssh/sshd_config
sed -i -e '/^\(#\|\)StrictModes/s/^.*$/StrictModes yes/' /etc/ssh/sshd_config
sed -i -e '/^\(#\|\)MaxAuthTries/s/^.*$/MaxAuthTries 3/' /etc/ssh/sshd_config
sed -i -e '/^\(#\|\)MaxSessions/s/^.*$/MaxSessions 2/' /etc/ssh/sshd_config
sed -i -e '/^\(#\|\)IgnoreRhosts/s/^.*$/IgnoreRhosts yes/' /etc/ssh/sshd_config
sed -i -e '/^\(#\|\)PermitEmptyPasswords/s/^.*$/PermitEmptyPasswords no/' /etc/ssh/sshd_config
sed -i -e '/^\(#\|\)UsePam/s/^.*$/UsePam yes/' /etc/ssh/sshd_config
sed -i -e '/^\(#\|\)AllowAgentForwarding/s/^.*$/AllowAgentForwarding no/' /etc/ssh/sshd_config
sed -i -e '/^\(#\|\)AllowTcpForwarding/s/^.*$/AllowTcpForwarding no/' /etc/ssh/sshd_config
sed -i -e '/^\(#\|\)X11Forwarding/s/^.*$/X11Forwarding no/' /etc/ssh/sshd_config
sed -i -e '/^\(#\|\)PrintLastLog/s/^.*$/PrintLastLog yes/' /etc/ssh/sshd_config
sed -i -e '/^\(#\|\)TCPKeepAlive/s/^.*$/TCPKeepAlive no/' /etc/ssh/sshd_config
sed -i -e '/^\(#\|\)Compression/s/^.*$/Compression no/' /etc/ssh/sshd_config
sed -i -e "/^\(#\|\)ClientAliveInterval/s/^.*$/ClientAliveInterval 300/" /etc/ssh/sshd_config
sed -i -e "/^\(#\|\)ClientAliveCountMax/s/^.*$/ClientAliveCountMax 1/" /etc/ssh/sshd_config
echo "AllowUsers $user" >> /etc/ssh/sshd_config
echo "Protocol 2" >> /etc/ssh/sshd_config

echo
echo "$(tput setaf 1)Adding SSH key for $user"$(tput setaf 7)
mkdir /home/$user/.ssh
chown $user:$user /home/$user/.ssh
chmod 700 /home/$user/.ssh
touch /home/$user/.ssh/authorized_keys
echo
chown $user:$user /home/$user/.ssh/authorized_keys
chmod 600 /home/$user/.ssh/authorized_keys
sed -i -e '/^\(#\|\)PasswordAuthentication/s/^.*$/PasswordAuthentication no/' /etc/ssh/sshd_config
echo "[insert your SSH public key here]" > /home/$user/.ssh/authorized_keys
echo "$(tput setaf 1)Restarting SSHD"$(tput setaf 7)
systemctl restart sshd

# Set up unattended-upgrades to install security updates
echo
echo "$(tput setaf 1)Setting up automatic security updates"$(tput setaf 7)
apt-get -option=Dpkg::options::=--force-confmiss --option=Dpkg::options::=--force-confold --assume-yes install unattended-upgrades apt-listchanges apticron

# Install UFW firewall (if uncommented make sure to comment out CSF firewall)
#echo
#echo "$(tput setaf 1)Setting up the UFW firewall"$(tput setaf 7)
#apt-get -option=Dpkg::options::=--force-confmiss --option=Dpkg::options::=--force-confold --assume-yes install ufw
#ufw default deny incoming
#ufw default allow outgoing
#ufw logging on
#ufw allow $port/tcp
#ufw limit $port/tcp
#ufw allow 53/tcp
#ufw allow 587/tcp
# only enable the next two if using Zabbix
#ufw allow 10050/tcp
#ufw allow 10051/tcp
#ufw enable

# Install CSF firewall (if uncommented make sure to comment out UFW firewall)
echo
echo "$(tput setaf 1)Setting up CSF"$(tput setaf 7)
apt-get -option=Dpkg::options::=--force-confmiss --option=Dpkg::options::=--force-confold --assume-yes remove ufw
apt-get -option=Dpkg::options::=--force-confmiss --option=Dpkg::options::=--force-confold --assume-yes install perl iptables libwww-perl liblwp-protocol-https-perl ipset ipset-persistent
cd /usr/src
wget https://download.configserver.com/csf.tgz
tar -xzf csf.tgz
cd csf
sh install.sh
sed -i "s/RESTRICT_SYSLOG\s*=.*$/RESTRICT_SYSLOG = \"3\"/g" /etc/csf/csf.conf
sed -i "s/SYSLOG_CHECK\s*=.*$/SYSLOG_CHECK = \"3600\"/g" /etc/csf/csf.conf
sed -i "s/SMTP_BLOCK\s*=.*/SMTP_BLOCK = \"0\"/g" /etc/csf/csf.conf
sed -i "s/LF_GLOBAL\s*=.*$/LF_GLOBAL = \"1800\"/g" /etc/csf/csf.conf
sed -i "s/LF_SCRIPT_ALERT\s*=.*$/LF_SCRIPT_ALERT = \"1\"/g" /etc/csf/csf.conf
sed -i "s/PT_ALL_USERS\s*=.*$/PT_ALL_USERS = \"1\"/g" /etc/csf/csf.conf
sed -i "s/TESTING = \"1\"/TESTING = \"0\"/g" /etc/csf/csf.conf
sed -i -e "/^\(\)TCP_IN/s/^.*$/TCP_IN = \"53,80,443,587,10050,10051,$port\"/" /etc/csf/csf.conf
sed -i -e "/^\(\)UDP_IN/s/^.*$/UDP_IN = \"53,80,443\"/" /etc/csf/csf.conf
sed -i -e "/^\(\)TCP_OUT/s/^.*$/TCP_OUT = \"53,80,443,587,10050,10051,$port\"/" /etc/csf/csf.conf
sed -i -e "/^\(\)UDP_OUT/s/^.*$/UDP_OUT = \"53,123\"/" /etc/csf/csf.conf
sed -i -e "/^\(\)TCP6_IN/s/^.*$/TCP6_IN = \"53,80,443,587,10050,10051,$port\"/" /etc/csf/csf.conf
sed -i -e "/^\(\)UDP6_IN/s/^.*$/UDP6_IN = \"53,80,443\"/" /etc/csf/csf.conf
sed -i -e "/^\(\)TCP6_OUT/s/^.*$/TCP6_OUT = \"53,80,443,587,10050,10051,$port\"/" /etc/csf/csf.conf
sed -i -e "/^\(\)UDP6_OUT/s/^.*$/UDP6_OUT = \"53,123\"/" /etc/csf/csf.conf
sed -i -e "/^\(\)IP6TABLES_RESTORE/s|^.*$|IP6TABLES_RESTORE = \"/usr/sbin/ip6tables-restore\"|" /etc/csf/csf.conf
sed -i -e "/^\(\)IP6TABLES_SAVE/s|^.*$|IP6TABLES_SAVE = \"/usr/sbin/ip6tables-save\"|" /etc/csf/csf.conf
sed -i -e "/^\(\)IP6TABLES = /s|^.*$|IP6TABLES = \"/usr/sbin/ip6tables\"|" /etc/csf/csf.conf
sed -i -e "/^\(\)IPTABLES_RESTORE/s|^.*$|IPTABLES_RESTORE = \"/usr/sbin/iptables-restore\"|" /etc/csf/csf.conf
sed -i -e "/^\(\)IPTABLES_SAVE/s|^.*$|IPTABLES_SAVE = \"/usr/sbin/iptables-save\"|" /etc/csf/csf.conf
sed -i -e "/^\(\)IPTABLES = /s|^.*$|IPTABLES = \"/usr/sbin/iptables\"|" /etc/csf/csf.conf
sed -i "s/CONNLIMIT = \"\"/CONNLIMIT = \"$port;3\"/g" /etc/csf/csf.conf

echo "exe:/usr/bin/freshclam" >> /etc/csf/csf.pignore
echo "exe:/usr/sbin/clamd" >> /etc/csf/csf.pignore
echo "exe:/usr/bin/htcacheclean" >> /etc/csf/csf.pignore
echo "exe:/usr/sbin/atd" >> /etc/csf/csf.pignore
echo "exe:/usr/sbin/rsyslogd" >> /etc/csf/csf.pignore
echo "exe:/usr/lib/systemd/systemd" >> /etc/csf/csf.pignore
echo "exe:/usr/lib/systemd/systemd-timesyncd" >> /etc/csf/csf.pignore
echo "exe:/usr/lib/systemd/systemd-networkd" >> /etc/csf/csf.pignore
echo "exe:/usr/lib/postfix/sbin/qmgr" >> /etc/csf/csf.pignore
echo "exe:/usr/lib/postfix/sbin/pickup" >> /etc/csf/csf.pignore
echo "exe:/usr/lib/postfix/sbin/tlsmgr" >> /etc/csf/csf.pignore
echo "exe:/usr/sbin/zabbix_agent2" >> /etc/csf/csf.pignore
# only adds the process for the selected CLI version of PHP
# echo "exe:/usr/sbin/php-fpm$(php -v | grep -Po '(?<=PHP )([0-9].[0-9])')" >> /etc/csf/csf.pignore

csf --profile apply protection_high
sed -i "s/LF_PERMBLOCK_ALERT = \"1\"/LF_PERMBLOCK_ALERT = \"0\"/g" /etc/csf/csf.conf
sed -i "s/LF_NETBLOCK_ALERT = \"1\"/LF_NETBLOCK_ALERT = \"0\"/g" /etc/csf/csf.conf
sed -i "s/LF_EMAIL_ALERT = \"1\"/LF_EMAIL_ALERT = \"0\"/g" /etc/csf/csf.conf
sed -i "s/LF_TEMP_EMAIL_ALERT = \"1\"/LF_TEMP_EMAIL_ALERT = \"0\"/g" /etc/csf/csf.conf
sed -i "s/CT_EMAIL_ALERT = \"1\"/CT_EMAIL_ALERT = \"0\"/g" /etc/csf/csf.conf
sed -i "s/PS_EMAIL_ALERT = \"1\"/PS_EMAIL_ALERT = \"0\"/g" /etc/csf/csf.conf
sed -i "s/AT_ALERT = \"1\"/AT_ALERT = \"0\"/g" /etc/csf/csf.conf
sed -i "s/AT_NEW = \"1\"/AT_NEW = \"0\"/g" /etc/csf/csf.conf
sed -i "s/AT_OLD = \"1\"/AT_OLD = \"0\"/g" /etc/csf/csf.conf
sed -i "s/AT_PASSWD = \"1\"/AT_PASSWD = \"0\"/g" /etc/csf/csf.conf
sed -i "s/AT_UID = \"1\"/AT_UID = \"0\"/g" /etc/csf/csf.conf
sed -i "s/AT_GID = \"1\"/AT_GID = \"0\"/g" /etc/csf/csf.conf
sed -i "s/AT_DIR = \"1\"/AT_DIR = \"0\"/g" /etc/csf/csf.conf
sed -i "s/AT_SHELL = \"1\"/AT_SHELL = \"0\"/g" /etc/csf/csf.conf
sed -i "s/LOGSCANNER = \"1\"/LOGSCANNER = \"0\"/g" /etc/csf/csf.conf

# Set up the syslog for CSF
sed -i '31s/.*/*.=info;*.notice;*.warn;\\/' /etc/rsyslog.d/50-default.conf
sed -i '32s/.*/auth,authpriv.none;\\/' /etc/rsyslog.d/50-default.conf
sed -i '33s/.*/cron,daemon.none;\\/' /etc/rsyslog.d/50-default.conf
sed -i '34s/.*/mail,news.none -\/var\/log\/messages\\/' /etc/rsyslog.d/50-default.conf
systemctl restart rsyslog
 
csf -ra

# Install and setup Fail2ban to protect SSH port
echo
echo "$(tput setaf 1)Setting up fail2ban"$(tput setaf 7)
apt-get -option=Dpkg::options::=--force-confmiss --option=Dpkg::options::=--force-confold --assume-yes install fail2ban
systemctl enable fail2ban
cp /etc/fail2ban/jail.{conf,local}
sed -i -e '/^\(\)bantime/s/^.*$/bantime = -1/' /etc/fail2ban/jail.local
sed -i -e '/^\(#\|\)findtime/s/^.*$/findtime = 10m/' /etc/fail2ban/jail.local
sed -i -e '/^\(#\|\)maxretry/s/^.*$/maxretry = 3/' /etc/fail2ban/jail.local
sed -i -e '/^\(#\|\)action = %(action/s/^.*$/action = %(action_)s/' /etc/fail2ban/jail.local
#sed -i -e '/^\(#\|\)destemail/s/^.*$/destemail = admin@philipstonemedia.com/' /etc/fail2ban/jail.local
#sed -i -e '/^\(#\|\)sender/s/^.*$/sender = admin@philipstonemedia.com/' /etc/fail2ban/jail.local
sed -i -e '/^\[sshd\]/s/^.*$/\[sshd\]\nenabled=true/' /etc/fail2ban/jail.local
sed -i -e "/^port    = ssh/s/^.*$/port    = $port/" /etc/fail2ban/jail.local
systemctl restart fail2ban

# Set up Logwatch to email reports daily
echo
echo "$(tput setaf 1)Setting up Logwatch"$(tput setaf 7)
apt-get -option=Dpkg::options::=--force-confmiss --option=Dpkg::options::=--force-confold --assume-yes install logwatch
mkdir /var/cache/logwatch
cp /usr/share/logwatch/default.conf/logwatch.conf /etc/logwatch/conf/
#sed -i -e '/^\(#\|\)MailTo = root/s/^.*$/MailTo = [insert email address here]/' /etc/logwatch/conf/logwatch.conf

# Set up SSMTP so we can send emails
#echo
#echo "$(tput setaf 1)Setting up SSMTP"$(tput setaf 7)
#apt-get -option=Dpkg::options::=--force-confmiss --option=Dpkg::options::=--force-confold --assume-yes install ssmtp mailutils
#rm /etc/ssmtp/ssmtp.conf
#touch /etc/ssmtp/ssmtp.conf
#echo "root=insert email address here" >> /etc/ssmtp/ssmtp.conf
#echo "mailhub=insert smtp_host here" >> /etc/ssmtp/ssmtp.conf
#echo "#rewriteDomain=" >> /etc/ssmtp/ssmtp.conf
#echo "hostname=$host" >> /etc/ssmtp/ssmtp.conf
#echo "FromLineOverride=YES" >> /etc/ssmtp/ssmtp.conf
#echo "AuthUser=insert email address here" >> /etc/ssmtp/ssmtp.conf
#echo "AuthPass=insert email password here" >> /etc/ssmtp/ssmtp.conf
#echo "UseTLS=YES" >> /etc/ssmtp/ssmtp.conf
#echo "UseSTARTTLS=YES" >> /etc/ssmtp/ssmtp.conf

# Set up postfix to relay emails
echo
echo "$(tput setaf 1)Setting up postfix"$(tput setaf 7)
apt-get -option=Dpkg::options::=--force-confmiss --option=Dpkg::options::=--force-confold --assume-yes install postfix libsasl2-modules mailutils
echo "relay_domains = " >> /etc/postfix/main.cf
echo "masquerade_domains = [insert email domain here]" >> /etc/postfix/main.cf
echo "smtp_generic_maps = hash:/etc/postfix/generic" >> /etc/postfix/main.cf
echo "# outbound relay configurations" >> /etc/postfix/main.cf
echo "smtp_sasl_auth_enable = yes" >> /etc/postfix/main.cf
echo "smtp_sasl_password_maps = hash:/etc/postfix/sasl_passwd" >> /etc/postfix/main.cf
echo "smtp_sasl_security_options = noanonymous" >> /etc/postfix/main.cf
echo "smtp_use_tls = yes" >> /etc/postfix/main.cf
echo "smtp_tls_CAfile = /etc/ssl/certs/ca-certificates.crt" >> /etc/postfix/main.cf
echo "smtp_tls_security_level = may" >> /etc/postfix/main.cf
echo "header_size_limit = 4096000" >> /etc/postfix/main.cf
sed -i -e '/^\(#\|\)relayhost =/s/^.*$/relayhost = [insert relay address here]/' /etc/postfix/main.cf
sed -i -e "/^\(#\|\)myhostname =/s/^.*$/myhostname = $shorthost/" /etc/postfix/main.cf
sed -i -e '/^\(#\|\)inet_interfaces =/s/^.*$/inet_interfaces = loopback-only/' /etc/postfix/main.cf
sed -i -e '/^\(#\|\)  -o smtpd_sender_restrictions=/s/^.*$/  -o smtpd_sender_restrictions=reject_unknown_sender_domain/' /etc/postfix/master.cf
sed -i -e '/^\(#\|\)  -o smtpd_recipient_restrictions=/s/^.*$/  -o smtpd_recipient_restrictions=permit_mynetworks, permit_sasl_authenticated, reject_unauth_destination/' /etc/postfix/master.cf

# Setup generic file to rewrite from addresses
echo "root@$host [insert your email address here]" > /etc/postfix/generic
echo "phlpstn@$host [insert your email address here]" >> /etc/postfix/generic
#echo "@$host [insert your email address here]" >> /etc/postfix/generic
#echo "@$hostdomain [insert your email address here]" >> /etc/postfix/generic
postmap /etc/postfix/generic

# Set up SMTP login details
echo "[insert email relay login here] [insert email relay password here]" > /etc/postfix/sasl_passwd
postmap /etc/postfix/sasl_passwd
systemctl restart postfix
chmod 0600 /etc/postfix/sasl_passwd /etc/postfix/sasl_passwd.db

# Install and setup the Zabbix agent. Needs setting up manually on the Zabbix server
#echo
#echo "$(tput setaf 1)Installing Zabbix agent"$(tput setaf 7)
#cd ~
#wget "https://repo.zabbix.com/zabbix/6.2/ubuntu/pool/main/z/zabbix-release/zabbix-release_6.2-4%2Bubuntu$(lsb_release -sr)_all.deb"
#dpkg -i "zabbix-release_6.2-4+ubuntu$(lsb_release -sr)_all.deb"
#apt update
#apt-get -option=Dpkg::options::=--force-confmiss --option=Dpkg::options::=--force-confold --assume-yes install zabbix-agent2 zabbix-agent2-plugin-*
#openssl rand -hex 256 > /etc/zabbix/$shorthost.psk
#sed -i -e '/^\(\)Server=/s/^.*$/Server=[insert zabbix host ip address here]/' /etc/zabbix/zabbix_agent2.conf
#sed -i -e '/^\(\)ServerActive=127.0.0.1/s/^.*$/[insert zabbix host ip address here]/' /etc/zabbix/zabbix_agent2.conf  

#sed -i -e '/^\(\)Hostname=/s/^.*$/Hostname='"${shorthost}"'/' /etc/zabbix/zabbix_agent2.conf
#sed -i -e '/^\(#\|\) TLSConnect=/s/^.*$/TLSConnect=psk/' /etc/zabbix/zabbix_agent2.conf
#sed -i -e '/^\(#\|\) TLSAccept=/s/^.*$/TLSAccept=psk/' /etc/zabbix/zabbix_agent2.conf
#sed -i -e '/^\(#\|\) TLSPSKIdentity=/s/^.*$/TLSPSKIdentity='"${shorthost}"'/' /etc/zabbix/zabbix_agent2.conf
#sed -i -e '/^\(#\|\) TLSPSKFile=/s/^.*$/TLSPSKFile=\/etc\/zabbix\/'"${shorthost}.psk"'/' /etc/zabbix/zabbix_agent2.conf
#systemctl restart zabbix-agent2
#systemctl enable zabbix-agent2
#rm "zabbix-release_6.2-4+ubuntu$(lsb_release -sr)_all.deb"

# Set up Lynis security checking tool
#echo
#echo "$(tput setaf 1)Setting up Lynis"$(tput setaf 7)
#apt-get -option=Dpkg::options::=--force-confmiss --option=Dpkg::options::=--force-confold --assume-yes install gnupg gnupg2
#wget -O - https://packages.cisofy.com/keys/cisofy-software-public.key | sudo apt-key add -
#echo "deb https://packages.cisofy.com/community/lynis/deb/ stable main" | sudo tee /etc/apt/sources.list.d/cisofy-lynis.list
#echo 'Acquire::Languages "none";' | sudo tee /etc/apt/apt.conf.d/99disable-translations
#apt update
#apt-get -option=Dpkg::options::=--force-confmiss --option=Dpkg::options::=--force-confold --assume-yes install lynis

# Setup Clamav
echo
echo "$(tput setaf 1)Setting up clamav"$(tput setaf 7)
apt-get -option=Dpkg::options::=--force-confmiss --option=Dpkg::options::=--force-confold --assume-yes install clamav-base clamav clamav-daemon clamav-freshclam clamdscan
systemctl stop clamav-freshclam
freshclam
systemctl start clamav-freshclam
apt-get purge -y clamav-unofficial-sigs
mkdir -p /usr/local/sbin/
wget https://raw.githubusercontent.com/extremeshok/clamav-unofficial-sigs/master/clamav-unofficial-sigs.sh -O /usr/local/sbin/clamav-unofficial-sigs.sh
chmod 755 /usr/local/sbin/clamav-unofficial-sigs.sh
mkdir -p /etc/clamav-unofficial-sigs/
wget https://raw.githubusercontent.com/extremeshok/clamav-unofficial-sigs/master/config/master.conf -O /etc/clamav-unofficial-sigs/master.conf
wget https://raw.githubusercontent.com/extremeshok/clamav-unofficial-sigs/master/config/user.conf -O /etc/clamav-unofficial-sigs/user.conf
wget https://raw.githubusercontent.com/extremeshok/clamav-unofficial-sigs/master/config/os/os.ubuntu.conf -O /etc/clamav-unofficial-sigs/os.conf
/usr/local/sbin/clamav-unofficial-sigs.sh --force
/usr/local/sbin/clamav-unofficial-sigs.sh --install-logrotate
/usr/local/sbin/clamav-unofficial-sigs.sh --install-man
mkdir -p /etc/systemd/system/
wget https://raw.githubusercontent.com/extremeshok/clamav-unofficial-sigs/master/systemd/clamav-unofficial-sigs.service -O /etc/systemd/system/clamav-unofficial-sigs.service
wget https://raw.githubusercontent.com/extremeshok/clamav-unofficial-sigs/master/systemd/clamav-unofficial-sigs.timer -O /etc/systemd/system/clamav-unofficial-sigs.timer
systemctl enable clamav-unofficial-sigs.service
systemctl enable clamav-unofficial-sigs.timer
systemctl start clamav-unofficial-sigs.timer

# Set up maldetect
echo
echo "$(tput setaf 1)Setting up maldetect"$(tput setaf 7)
cd ~
wget https://www.rfxn.com/downloads/maldetect-current.tar.gz
mkdir maldetect
tar -xvf maldetect-current.tar.gz -C maldetect --strip-components=1
cd maldetect
./install.sh
sed -i -e '/^\(#\|\)email_alert="0"/s/^.*$/email_alert="1"/' /usr/local/maldetect/conf.maldet
sed -i -e '/^\(#\|\)email_addr="you@domain.com"/s/^.*$/email_addr="[insert your email address here]"/' /usr/local/maldetect/conf.maldet
sed -i -e '/^\(#\|\)scan_user_access="0"/s/^.*$/scan_user_access="1"/' /usr/local/maldetect/conf.maldet
sed -i -e '/^\(#\|\)scan_ignore_root="1"/s/^.*$/scan_ignore_root="0"/' /usr/local/maldetect/conf.maldet
/usr/local/sbin/maldet --mkpubpaths
maldet -u
maldet -d
cd ~
rm -rf maldetect
rm maldetect-current.tar.gz

# Install Crowdsec
echo
echo "$(tput setaf 1)Setting up Crowdsec"$(tput setaf 7)
cd ~
curl -s https://packagecloud.io/install/repositories/crowdsec/crowdsec/script.deb.sh | bash
apt-get -option=Dpkg::options::=--force-confmiss --option=Dpkg::options::=--force-confold --assume-yes install crowdsec
sed -i -e "/^  type: sqlite/s/^.*$/  type: sqlite\n  use_wal: true/" /etc/crowdsec/config.yaml

if [ $uver \< 22 ]
then
  apt-get -option=Dpkg::options::=--force-confmiss --option=Dpkg::options::=--force-confold --assume-yes install crowdsec-firewall-bouncer-iptables
else
  apt-get -option=Dpkg::options::=--force-confmiss --option=Dpkg::options::=--force-confold --assume-yes install crowdsec-firewall-bouncer-nftables
fi

cscli postoverflows install crowdsecurity/seo-bots-whitelist

systemctl reload crowdsec
systemctl enable crowdsec

# Apply any new updates
echo
echo "$(tput setaf 1)Updating system. Please wait..."$(tput setaf 7)
apt-get update
apt-get -option=Dpkg::options::=--force-confmiss --option=Dpkg::options::=--force-confold --assume-yes upgrade

# Clean up
echo
echo "$(tput setaf 1)Cleaning up..."$(tput setaf 7)
apt autoremove -y
apt clean -y

# Finished - provide details
echo
echo "$(tput setaf 1)Finished - Please read!"$(tput setaf 7)
echo
echo "Hostname set to "$host
echo "User" $user "setup and added to the sudo group"
echo "SSH port is set to" $port". SSH access is only allowed by key"
echo 
echo "Crowdsec is installed. Make sure to run $(tput setaf 1)sudo /usr/share/crowdsec/wizard.sh -c$(tput setaf 7)"
echo "and add any new bouncers necessary if new services / software is installed"
echo
#echo "The Zabbix agent is set up on this server. To add it to the Zabbix server you will need:"
#echo
#echo "$(tput setaf 1)Hostname: $(tput setaf 7)$shorthost"
#echo "$(tput setaf 1)Templates: $(tput setaf 7)Templates/Operating systems then Linux by Zabbix agent"
#echo "$(tput setaf 1)Host Groups: $(tput setaf 7)Linux servers"
#echo "$(tput setaf 1)Interfaces / IP Address: $(tput setaf 7) $myv4 or $myv6"
#if [[ -z "$myv6" ]]; then
#  echo "$(tput setaf 1)Interfaces / IP Address: $(tput setaf 7) $myv4"
#else
#  echo "$(tput setaf 1)Interfaces / IP Address: $(tput setaf 7) $myv6"
#fi
#echo "$(tput setaf 1)Connections to host: $(tput setaf 7)PSK"
#echo "$(tput setaf 1)Connections from host: $(tput setaf 7)PSK"
#echo "$(tput setaf 1)PSK Identity: $(tput setaf 7)$shorthost"
#echo "$(tput setaf 1)PSK: $(tput setaf 7)"
#cat /etc/zabbix/$shorthost.psk
#echo
echo "$(tput setaf 1)Note: "$(tput setaf 7)"a reboot will be required"
echo